#ifndef Handler3DH //To avoid linker errors, define header's unique "name", so...
#define Handler3DH //...this header will not be compilled more than one time
//==============================================================================
#include <gl\gl.h>      // Header file for the main OpenGL library
#include <gl\glu.h>     // Header file for the GLu library
//------------------------------------------------------------------------------
#include <windef.h>     // System header file
#include <wingdi.h>     // System header file
//==============================================================================

//Class contains functions for easy managing OpenGl window
class TOpenGLHandler{
//------------------------------------------------------------------------------
//Drawing object' handlers
HDC hDC;    //Devise context' handler
HGLRC hRC;  //Render context' handler
void* hWnd; //Drawing object' handler
//Perspective matrix setup parameters
float PerspectiveAngle;  //View angle
float AspectRatio;       //AspectRatio = ViewportWidth / ViewportHeight
float NearClippingPlane; //Primitives which closer to observer are clipped
float FarClippingPlane;  //Maximum view distance
//------------------------------------------------------------------------------

public:
//Default class' constructor (OpenGL is NOT initialized here!)
TOpenGLHandler(){
 //Set defaults
 hDC = NULL;
 hRC = NULL;
 hWnd = NULL;
 PerspectiveAngle=45.f;
 AspectRatio=1.f;
 NearClippingPlane=0.1f;
 FarClippingPlane=1000.f;
}

//Default class' destructor; OpenGL context will be terminated automatically
~TOpenGLHandler(){
 wglMakeCurrent(NULL, NULL);
 wglDeleteContext(hRC);
 ReleaseDC(hWnd,hDC);
}       
//------------------------------------------------------------------------------

//Function initializes OpenGL with defined parameters; you need to specify
//handle of the object, in which 3D drawing will be performed
void InitOpenGL(void *DrawingObjectHandle,
                int ViewportWidth,
                int ViewportHeight){
 //Store handle
 hWnd = DrawingObjectHandle;
  //Get device context
 hDC = GetDC(DrawingObjectHandle);
 //Mark our preferences concerning pixel format
 PIXELFORMATDESCRIPTOR pfd = {
   sizeof(PIXELFORMATDESCRIPTOR),  // size of this pfd
   1,                     // version number
   PFD_DRAW_TO_WINDOW |   // support window
   PFD_SUPPORT_OPENGL |   // support OpenGL
   PFD_DOUBLEBUFFER,      // double buffered
   PFD_TYPE_RGBA,         // RGBA type
   24,                    // 24-bit color depth
   0, 0, 0, 0, 0, 0,      // color bits ignored
   0,                     // no alpha buffer
   0,                     // shift bit ignored
   0,                     // no accumulation buffer
   0, 0, 0, 0,            // accum bits ignored
   32,                    // 32-bit depth-buffer
   8,                     // 8-bit stencil buffer
   0,                     // no auxiliary buffer
   PFD_MAIN_PLANE,        // main layer
   0,                     // reserved
   0, 0, 0                // layer masks ignored
 };
 //Setup pixel format which is closest to choosen
 SetPixelFormat(hDC, ChoosePixelFormat(hDC, &pfd), &pfd);
 //Initialize OpenGL context
 hRC = wglCreateContext(hDC); //Create OpenGL context
 wglMakeCurrent(hDC, hRC);    //Make this context current for OpenGL functions
  //Define viewport size
 glViewport(0,0, ViewportWidth,ViewportHeight);
 //Calculate and store viewport-s aspect ratio
 AspectRatio=float(ViewportWidth)/float(ViewportHeight);
 //Calculate perspective matrix
 SetupPerspective(PerspectiveAngle,NearClippingPlane,FarClippingPlane);
}      
//------------------------------------------------------------------------------

//Call this function to define perspective settings (default: 45, 0.1, 1000)
inline void SetupPerspective(float ViewAngle,
                             float MinViewDistance,
                             float MaxViewDistance){
 PerspectiveAngle=ViewAngle;
 NearClippingPlane=MinViewDistance;
 FarClippingPlane=MaxViewDistance;
 glMatrixMode(GL_PROJECTION); glLoadIdentity(); //Clear projection matrix
 //Configure perspective
 gluPerspective(PerspectiveAngle,   //Perspective angle
                AspectRatio,        //Viewport aspect ratio
                NearClippingPlane,  //Distance to front clipping plane
                FarClippingPlane);  //Distance to far clipping plane
 //Choose modelview matrix again and clear it
 glMatrixMode(GL_MODELVIEW); glLoadIdentity();
 //Refresh viewport
 SwapBuffers(hDC);
}
//------------------------------------------------------------------------------

//Call this function after drawing object's resizing to resize 3D viewport also
inline void ResizeViewport(int Width, int Height){
 glViewport(0,0, Width,Height);          //Resize viewport
 AspectRatio=float(Width)/float(Height); //Calculate and store aspect ratio
 //Recalculate perspective matrix
 SetupPerspective(PerspectiveAngle,NearClippingPlane,FarClippingPlane);
}

/*============================================================================*\
//Function changes OpenGL defaults after it's initialization. You are free to
//change them if your have different preferences
\*============================================================================*/
void SetupMySettings(){
 glEnable(GL_DEPTH_TEST);  //Enable depth-testing
 glDepthFunc(GL_LEQUAL);   //Draw closer and coinciding surfaces
 glClearDepth(1);          //Default value for clearing depth buffer
 glEnable(GL_STENCIL_TEST);           //Enable stencil test by default
 glStencilFunc(GL_EQUAL,1,1);         //Draw pixel only if stencil is equal to 1
 glStencilOp(GL_KEEP,GL_KEEP,GL_KEEP);//Do not change stencil buffer values
 glClearStencil(1);                   //Default value for stencil buffer
 glEnable(GL_ALPHA_TEST);  //Enable testing of alpha-component
 glAlphaFunc(GL_GREATER,0);//Draw only if pixel not fully-transparent (Alpha=0)
 glClearColor(0,0,0,1);    //Default value for clearing color buffer (black)
 glEnable(GL_BLEND);                              //Enable color blending
 glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);//Setup blending algorithm
 glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);//Draw polygons as surfaces
 glFrontFace(GL_CCW);    //Choose order of vertex passing
 glEnable(GL_CULL_FACE); //Do not draw...
 glCullFace(GL_BACK);    //...back sides
 glShadeModel(GL_SMOOTH);      //Choose smoothing for polygons
 glDisable(GL_COLOR_MATERIAL); //Disable glColor but enable glMaterial fully
 //Lightning settings
 glEnable(GL_LIGHTING);
 float lmAmb[4] = {0,0,0,1}; //Initial value: 0.2, 0.2, 0.2, 1
 glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmAmb);
 //glLightModeli(GL_LIGHT_MODEL_COLOR_CONTROL, GL_SINGLE_COLOR); //Or "GL_SEPARATE_SPECULAR"
 glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, 0); //Try "1"
 glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, 0);     //This is default value
 //Configure default lighter
 glEnable(GL_LIGHT0);
 float lAmb[4]={ 0.2, 0.2, 0.2, 1.0}; glLightfv(GL_LIGHT0,GL_AMBIENT,lAmb);
 float lDif[4]={ 1.0, 1.0, 1.0, 1.0}; glLightfv(GL_LIGHT0,GL_DIFFUSE,lDif);
 float lSpe[4]={ 1.0, 1.0, 1.0, 1.0}; glLightfv(GL_LIGHT0,GL_SPECULAR,lSpe);
 float lPos[4]={ 1.0, 1.0, 1.0, 0.0}; glLightfv(GL_LIGHT0,GL_POSITION,lPos);
 float lDir[3]={-1.0,-1.0,-1.0}; glLightfv(GL_LIGHT0,GL_SPOT_DIRECTION,lDir);
 glLighti(GL_LIGHT0,GL_SPOT_EXPONENT,0);
 glLighti(GL_LIGHT0,GL_SPOT_CUTOFF,180);
 glLightf(GL_LIGHT0,GL_LINEAR_ATTENUATION,1);
 glLighti(GL_LIGHT0,GL_QUADRATIC_ATTENUATION,0);
 glLighti(GL_LIGHT0,GL_CONSTANT_ATTENUATION,0);
}
//------------------------------------------------------------------------------

//Call this function to clear all OpenGL buffers and current modelview matrix
inline void FrameBegin(){
 glClear(GL_COLOR_BUFFER_BIT |
         GL_DEPTH_BUFFER_BIT |
         GL_ACCUM_BUFFER_BIT |
         GL_STENCIL_BUFFER_BIT);
 glLoadIdentity();
}

//Call this function to show drawn image
inline void FrameFinish(){
 SwapBuffers(hDC);
} 
//------------------------------------------------------------------------------
};
//==============================================================================

TOpenGLHandler Window3D;
extern TOpenGLHandler Window3D;

//==============================================================================
#endif //End header
